import { MDXProvider } from '@mdx-js/react';
import React, { createContext, useContext, useState, useEffect, useLayoutEffect } from 'react';
import merge from 'lodash.merge';
import { jsx, Global, ThemeContext } from '@emotion/core';
import styled from '@emotion/styled';
import { space, color, layout, flexbox } from 'styled-system';
import css, { css as css$1, get } from '@styled-system/css';
export { css } from '@styled-system/css';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var jsx$1 = function jsx$$1(type, props) {
  var arguments$1 = arguments;

  for (var _len = arguments.length, children = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    children[_key - 2] = arguments$1[_key];
  }

  return jsx.apply(undefined, [type, props ? _objectSpread({}, props, {
    css: props.css ? css(props.css) : undefined
  }) : null].concat(children));
};

var themed = function themed(key) {
  return function (props) {
    return css$1(get(props.theme, "styles.".concat(key)))(props.theme);
  };
};

var tags = ['p', 'b', 'i', 'a', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'img', 'pre', 'code', 'ol', 'ul', 'li', 'blockquote', 'hr', 'em', 'table', 'tr', 'th', 'td', 'em', 'strong', 'delete', // mdx
'inlineCode', 'thematicBreak', // other
'div', // theme-ui
'root'];
var aliases = {
  inlineCode: 'code',
  thematicBreak: 'hr',
  root: 'div'
};

var alias = function alias(n) {
  return aliases[n] || n;
};

var Styled = styled('div')(themed('div'), space);
var components = {};
tags.forEach(function (tag) {
  components[tag] = styled(alias(tag))(themed(tag), space);
  Styled[tag] = components[tag];
});

var Context = createContext({
  theme: {},
  components: components
});
var useThemeUI = function useThemeUI() {
  return useContext(Context);
};

var STORAGE_KEY = 'theme-ui-color-mode';
var storage = {
  get: function get$$1(init) {
    return window.localStorage.getItem(STORAGE_KEY) || init;
  },
  set: function set(value) {
    return window.localStorage.setItem(STORAGE_KEY, value);
  }
};
var getMediaQuery = function getMediaQuery() {
  var darkQuery = '(prefers-color-scheme: dark)';
  var mql = window.matchMedia ? window.matchMedia(darkQuery) : {};
  var dark = mql.media === darkQuery;
  return dark && mql.matches;
};
var useColorState = function useColorState(initialMode) {
  var _useState = useState(initialMode),
      _useState2 = _slicedToArray(_useState, 2),
      mode = _useState2[0],
      setMode = _useState2[1];

  useLayoutEffect(function () {
    // initialize
    var stored = storage.get();
    var dark = getMediaQuery();

    if (dark) {
      setMode('dark');
      return;
    }

    document.body.classList.remove('theme-ui-' + stored);
    if (!stored || stored === mode) { return; }
    setMode(stored);
  }, []);
  useEffect(function () {
    if (!mode) { return; }
    storage.set(mode);
  }, [mode]);
  return [mode, setMode];
};
var useColorMode = function useColorMode(initialMode) {
  var _useThemeUI = useThemeUI(),
      colorMode = _useThemeUI.colorMode,
      setColorMode = _useThemeUI.setColorMode;

  if (typeof setColorMode !== 'function') {
    throw new Error("[useColorMode] requires the ThemeProvider component");
  }

  return [colorMode, setColorMode];
};

var bodyColor = function bodyColor() {
  var theme = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  if (!theme.colors || !theme.colors.modes) { return; }
  var modes = theme.colors.modes;
  var styles = {};
  Object.keys(modes).forEach(function (mode) {
    var colors = modes[mode];
    styles["&.theme-ui-".concat(mode)] = {
      color: colors.text,
      bg: colors.background
    };
  });
  return css$1({
    body: _objectSpread({}, styles, {
      color: 'text',
      bg: 'background'
    })
  })(theme);
};

var ColorMode = function ColorMode() {
  return React.createElement(Global, {
    styles: bodyColor
  });
};

var createComponents = function createComponents() {
  var components = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var next = {};
  Object.keys(components).forEach(function (key) {
    next[key] = styled(components[key])(themed(key));
  });
  return next;
};

var ThemeProvider = function ThemeProvider(_ref) {
  var theme = _ref.theme,
      components = _ref.components,
      props = _objectWithoutProperties(_ref, ["theme", "components"]);

  var outer = useThemeUI();
  var initialColorMode = outer.colorMode || (theme ? theme.initialColorMode : undefined);

  var _useColorState = useColorState(initialColorMode),
      _useColorState2 = _slicedToArray(_useColorState, 2),
      colorMode = _useColorState2[0],
      setColorMode = _useColorState2[1];

  var context = merge({}, {
    colorMode: colorMode,
    setColorMode: setColorMode
  }, outer, {
    theme: theme,
    components: createComponents(components)
  });

  if (context.colorMode) {
    var modes = get(context.theme, 'colors.modes', {});
    context.theme = merge({}, context.theme, {
      colors: get(modes, context.colorMode, context.theme.colors)
    });
  }

  return jsx$1(ThemeContext.Provider, {
    value: context.theme
  }, jsx$1(MDXProvider, {
    components: context.components
  }, jsx$1(Context.Provider, {
    value: context,
    children: props.children
  })));
};

var Box = styled('div')(css({
  boxSizing: 'border-box',
  minWidth: 0
}), space, color, layout, flexbox);
var Flex = styled(Box)({
  display: 'flex'
}); // root/page layout

var Layout = function Layout(props) {
  return jsx$1(Box, _objectSpread({}, props, {
    css: {
      minHeight: '100vh',
      display: 'flex',
      flexDirection: 'column',
      variant: 'styles.Layout'
    }
  }));
};
var Header = function Header(props) {
  return jsx$1(Box, _objectSpread({
    as: 'header'
  }, props, {
    css: {
      display: 'flex',
      variant: 'styles.Header'
    }
  }));
};
var Main = function Main(props) {
  return jsx$1(Box, _objectSpread({}, props, {
    css: {
      flex: '1 1 auto',
      variant: 'styles.Main'
    }
  }));
};
var Container = function Container(props) {
  return jsx$1(Box, _objectSpread({}, props, {
    css: {
      width: '100%',
      minWidth: 0,
      maxWidth: 1024,
      mx: 'auto',
      p: 4,
      variant: 'styles.Container'
    }
  }));
};
var Footer = function Footer(props) {
  return jsx$1(Box, _objectSpread({
    as: 'footer'
  }, props, {
    css: {
      display: 'flex',
      variant: 'styles.Footer'
    }
  }));
};

export { jsx$1 as jsx, ThemeProvider, Context, useThemeUI, ColorMode, useColorMode, Styled, Box, Flex, Layout, Header, Main, Container, Footer, ThemeProvider as ColorModeProvider, ThemeProvider as ComponentProvider };
//# sourceMappingURL=index.esm.js.map
